@@ -51,6 +51,7 @@
 
 struct GsdClipboardManagerPrivate
 {
+        guint    start_idle_id;
         Display *display;
         Window   window;
         Time     timestamp;
@@ -151,7 +152,7 @@ send_selection_notify (GsdClipboardManager *manager,
                     (XEvent *)&notify);
         XSync (manager->priv->display, False);
 
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 }
 
 static void
@@ -178,7 +179,7 @@ finish_selection_request (GsdClipboardManager *manager,
                     False, NoEventMask, (XEvent *) &notify);
         XSync (manager->priv->display, False);
 
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 }
 
 static int
@@ -554,7 +555,7 @@ convert_clipboard_target (IncrConversion      *rdata,
 
                         XSync (manager->priv->display, False);
 
-                        gdk_error_trap_pop ();
+                        gdk_error_trap_pop_ignored ();
                 }
         }
 }
@@ -825,11 +826,11 @@ clipboard_manager_watch_cb (GsdClipboardManager *manager,
         GdkDisplay *display;
 
         display = gdk_display_get_default ();
-        gdkwin = gdk_window_lookup_for_display (display, window);
+        gdkwin = gdk_x11_window_lookup_for_display (display, window);
 
         if (is_start) {
                 if (gdkwin == NULL) {
-                        gdkwin = gdk_window_foreign_new_for_display (display, window);
+                        gdkwin = gdk_x11_window_foreign_new_for_display (display, window);
                 } else {
                         g_object_ref (gdkwin);
                 }
@@ -920,6 +921,8 @@ start_clipboard_idle_cb (GsdClipboardManager *manager)
 
         gnome_settings_profile_end (NULL);
 
+        manager->priv->start_idle_id = 0;
+
         return FALSE;
 }
 
@@ -929,7 +932,7 @@ gsd_clipboard_manager_start (GsdClipboardManager *manager,
 {
         gnome_settings_profile_start (NULL);
 
-        g_idle_add ((GSourceFunc) start_clipboard_idle_cb, manager);
+        manager->priv->start_idle_id = g_idle_add ((GSourceFunc) start_clipboard_idle_cb, manager);
 
         gnome_settings_profile_end (NULL);
 
@@ -941,18 +944,27 @@ gsd_clipboard_manager_stop (GsdClipboardManager *manager)
 {
         g_debug ("Stopping clipboard manager");
 
-        clipboard_manager_watch_cb (manager,
-                                    manager->priv->window,
-                                    FALSE,
-                                    0,
-                                    NULL);
-        XDestroyWindow (manager->priv->display, manager->priv->window);
+        if (manager->priv->window != None) {
+                clipboard_manager_watch_cb (manager,
+                                            manager->priv->window,
+                                            FALSE,
+                                            0,
+                                            NULL);
+                XDestroyWindow (manager->priv->display, manager->priv->window);
+                manager->priv->window = None;
+        }
 
-        list_foreach (manager->priv->conversions, (Callback) conversion_free, NULL);
-        list_free (manager->priv->conversions);
+        if (manager->priv->conversions != NULL) {
+                list_foreach (manager->priv->conversions, (Callback) conversion_free, NULL);
+                list_free (manager->priv->conversions);
+                manager->priv->conversions = NULL;
+        }
 
-        list_foreach (manager->priv->contents, (Callback) target_data_unref, NULL);
-        list_free (manager->priv->contents);
+        if (manager->priv->contents != NULL) {
+                list_foreach (manager->priv->contents, (Callback) target_data_unref, NULL);
+                list_free (manager->priv->contents);
+                manager->priv->contents = NULL;
+        }
 }
 
 static void
@@ -961,10 +973,6 @@ gsd_clipboard_manager_set_property (GObject        *object,
                                     const GValue   *value,
                                     GParamSpec     *pspec)
 {
-        GsdClipboardManager *self;
-
-        self = GSD_CLIPBOARD_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -978,10 +986,6 @@ gsd_clipboard_manager_get_property (GObject        *object,
                                     GValue         *value,
                                     GParamSpec     *pspec)
 {
-        GsdClipboardManager *self;
-
-        self = GSD_CLIPBOARD_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -995,9 +999,6 @@ gsd_clipboard_manager_constructor (GType                  type,
                                    GObjectConstructParam *construct_properties)
 {
         GsdClipboardManager      *clipboard_manager;
-        GsdClipboardManagerClass *klass;
-
-        klass = GSD_CLIPBOARD_MANAGER_CLASS (g_type_class_peek (GSD_TYPE_CLIPBOARD_MANAGER));
 
         clipboard_manager = GSD_CLIPBOARD_MANAGER (G_OBJECT_CLASS (gsd_clipboard_manager_parent_class)->constructor (type,
                                                                                                       n_construct_properties,
@@ -1009,10 +1010,6 @@ gsd_clipboard_manager_constructor (GType                  type,
 static void
 gsd_clipboard_manager_dispose (GObject *object)
 {
-        GsdClipboardManager *clipboard_manager;
-
-        clipboard_manager = GSD_CLIPBOARD_MANAGER (object);
-
         G_OBJECT_CLASS (gsd_clipboard_manager_parent_class)->dispose (object);
 }
 
@@ -1051,6 +1048,9 @@ gsd_clipboard_manager_finalize (GObject *object)
 
         g_return_if_fail (clipboard_manager->priv != NULL);
 
+        if (clipboard_manager->priv->start_idle_id !=0)
+                g_source_remove (clipboard_manager->priv->start_idle_id);
+
         G_OBJECT_CLASS (gsd_clipboard_manager_parent_class)->finalize (object);
 }
 
