@@ -348,7 +348,7 @@ set_left_handed (GsdMouseManager *manager,
                 return;
 
         xdevice = open_gdk_device (device);
-        if (!xdevice)
+        if (xdevice == NULL)
                 return;
 
         /* If the device is a touchpad, swap tap buttons
@@ -406,14 +406,14 @@ set_motion (GsdMouseManager *manager,
         guint i;
 
         xdevice = open_gdk_device (device);
+        if (xdevice == NULL)
+                return;
+
         if (device_is_touchpad (xdevice))
                 settings = manager->priv->touchpad_settings;
         else
                 settings = manager->priv->mouse_settings;
 
-        if (xdevice == NULL)
-                return;
-
         /* Calculate acceleration */
         motion_acceleration = g_settings_get_double (settings, KEY_MOTION_ACCELERATION);
 
@@ -447,6 +447,8 @@ set_motion (GsdMouseManager *manager,
 
         /* Get the list of feedbacks for the device */
         states = XGetFeedbackControl (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice, &num_feedbacks);
+        if (states == NULL)
+                num_feedbacks = 0;
         state = (XFeedbackState *) states;
         for (i = 0; i < num_feedbacks; i++) {
                 if (state->class == PtrFeedbackClass) {
@@ -485,6 +487,7 @@ set_middle_button (GsdMouseManager *manager,
         int format;
         unsigned long nitems, bytes_after;
         unsigned char *data;
+        int rc;
 
         prop = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                             "Evdev Middle Button Emulation", True);
@@ -498,26 +501,23 @@ set_middle_button (GsdMouseManager *manager,
 
         gdk_error_trap_push ();
 
-        XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+        rc = XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                             xdevice, prop, 0, 1, False, XA_INTEGER, &type, &format,
                             &nitems, &bytes_after, &data);
 
-        if ((gdk_error_trap_pop () != 0)) {
-                XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
-                return;
-        }
-
-        if (format == 8 && type == XA_INTEGER && nitems == 1) {
+        if (rc == Success && format == 8 && type == XA_INTEGER && nitems == 1) {
                 data[0] = middle_button ? 1 : 0;
 
-                gdk_error_trap_push ();
                 XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                                        xdevice, prop, type, format, PropModeReplace, data, nitems);
-
-                gdk_error_trap_pop_ignored ();
         }
 
-        XFree (data);
+        if (gdk_error_trap_pop ())
+                g_warning ("Error in setting middle button emulation on \"%s\"", gdk_device_get_name (device));
+
+        if (rc == Success)
+                XFree (data);
+
         XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
 }
 
@@ -576,6 +576,8 @@ set_tap_to_click (GdkDevice *device,
                 return;
 
         xdevice = open_gdk_device (device);
+        if (xdevice == NULL)
+                return;
 
         if (!device_is_touchpad (xdevice)) {
                 XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
@@ -598,10 +600,11 @@ set_tap_to_click (GdkDevice *device,
 
         if (rc == Success)
                 XFree (data);
-        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
-        if (gdk_error_trap_pop ()) {
+
+        if (gdk_error_trap_pop ())
                 g_warning ("Error in setting tap to click on \"%s\"", gdk_device_get_name (device));
-        }
+
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
 }
 
 static void
@@ -622,6 +625,8 @@ set_horiz_scroll (GdkDevice *device,
                 return;
 
         xdevice = open_gdk_device (device);
+        if (xdevice == NULL)
+                return;
 
         if (!device_is_touchpad (xdevice)) {
                 XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
@@ -655,11 +660,14 @@ set_horiz_scroll (GdkDevice *device,
                                        PropModeReplace, data, nitems);
         }
 
-        XFree (data);
-        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
-
         if (gdk_error_trap_pop ())
                 g_warning ("Error in setting horiz scroll on \"%s\"", gdk_device_get_name (device));
+
+        if (rc == Success)
+                XFree (data);
+
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
+
 }
 
 static void
@@ -680,6 +688,8 @@ set_edge_scroll (GdkDevice               *device,
                 return;
 
         xdevice = open_gdk_device (device);
+        if (xdevice == NULL)
+                return;
 
         if (!device_is_touchpad (xdevice)) {
                 XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
@@ -713,11 +723,13 @@ set_edge_scroll (GdkDevice               *device,
                                        PropModeReplace, data, nitems);
         }
 
-        XFree (data);
-        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
-
         if (gdk_error_trap_pop ())
                 g_warning ("Error in setting edge scroll on \"%s\"", gdk_device_get_name (device));
+
+        if (rc == Success)
+                XFree (data);
+
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
 }
 
 static void
@@ -734,6 +746,8 @@ set_touchpad_enabled (GdkDevice *device,
                 return;
 
         xdevice = open_gdk_device (device);
+        if (xdevice == NULL)
+                return;
 
         if (!device_is_touchpad (xdevice)) {
                 XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
@@ -744,14 +758,11 @@ set_touchpad_enabled (GdkDevice *device,
         XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice,
                                prop_enabled, XA_INTEGER, 8,
                                PropModeReplace, &data, 1);
-        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
-        gdk_flush ();
 
-        if (gdk_error_trap_pop ()) {
-                g_warning ("Error %s device \"%s\"",
-                           (state) ? "enabling" : "disabling",
-                           gdk_device_get_name (device));
-        }
+        if (gdk_error_trap_pop ())
+                g_warning ("Error %s device \"%s\"", (state) ? "enabling" : "disabling", gdk_device_get_name (device));
+
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
 }
 
 static void
